# !/usr/bin/python
# coding=utf-8
import os
import unittest
import inspect
import pymel.core as pm
from mayatk import XformUtils


# sfr = pm.melGlobals['cmdScrollFieldReporter']
# pm.cmdScrollFieldReporter(sfr, edit=1, clear=1)


class Main(unittest.TestCase):
    """Main test class."""

    def perform_test(self, cases):
        """Execute the test cases."""
        for case in cases:
            if isinstance(case, str):
                expression = case
                expected_result = cases[case]
                method_name = str(expression).split("(")[0]
            else:
                result, expected_result = case
                method_name = result.__class__.__name__
                expression = None

            try:
                path = os.path.abspath(inspect.getfile(eval(method_name)))
            except (TypeError, IOError):
                path = ""

            if expression:
                result = eval(expression)

            self.assertEqual(
                result,
                expected_result,
                f"\n\n# Error: {path}\n#\tCall: {method_name})\n#\tExpected {type(expected_result)}: {expected_result}\n#\tReturned {type(result)}: {result}",
            )


class XformUtils_test(unittest.TestCase, XformUtils):
    """Unit tests for the XformUtils class"""

    def setUp(self):
        """Set up test scene"""
        pm.mel.file(new=True, force=True)
        self.cube1 = pm.polyCube(name="cube1")[0]
        self.cube2 = pm.polyCube(name="cube2")[0]
        self.sph = pm.polySphere(name="sph")[0]

    def tearDown(self):
        """Clean up test scene"""
        pm.delete(self.cube1, self.cube2, self.sph)

    def test_moveTo(self):
        self.assertEqual(self.move_to(self.cube1, self.cube2), None)

    def test_dropToGrid(self):
        self.assertEqual(
            self.drop_to_grid(
                self.cube1,
                align="Min",
                origin=True,
                center_pivot=True,
                freeze_transforms=True,
            ),
            None,
        )

    def test_resetTranslation(self):
        self.assertEqual(self.reset_translation(self.cube1), None)

    def test_setTranslationToPivot(self):
        self.assertEqual(self.set_translation_to_pivot(self.cube1), None)

    def test_alignPivotToSelection(self):
        self.assertEqual(self.align_pivot_to_selection(self.cube1, self.cube2), None)

    def test_aimObjectAtPoint(self):
        self.assertEqual(
            self.aim_object_at_point([self.cube1, self.cube2], (0, 15, 15)), None
        )

    def test_rotateAxis(self):
        self.assertEqual(self.rotate_axis([self.cube1, self.cube2], (0, 15, 15)), None)

    def test_getOrientation(self):
        self.assertEqual(
            self.get_orientation(self.cube1), ([1, 0, 0], [0, 1, 0], [0, 0, 1])
        )

    def test_getDistanceBetweenTwoObjects(self):
        self.drop_to_grid([self.cube1, self.cube2], origin=True, center_pivot=True)
        pm.move(self.cube2, 0, 0, 15)
        self.assertEqual(self.get_dist_between_two_objects(self.cube1, self.cube2), 15)

    def test_getCenterPoint(self):
        self.assertEqual(self.get_center_point(self.sph), (0, 0, 0))

    def test_getBoundingBox(self):
        self.assertEqual(self.get_bounding_box(self.sph, "size"), (10, 10, 10))

    def test_sortByBoundingBoxValue(self):
        self.assertEqual(
            str(self.sort_by_bounding_box_value(["sph.vtx[0]", "sph.f[0]"])),
            "[MeshFace('sphShape.f[0]'), MeshVertex('sphShape.vtx[0]')]",
        )

    def test_matchScale(self):
        self.assertEqual(
            self.match_scale(self.cube1, self.cube2, scale=False),
            [1.3063946090989371, 0.539387725343009, 0.539387708993454],
        )

    def test_snap3PointsTo3Points(self):
        """ """
        # self.assertEqual(self.align_using_three_points(), None)

    def test_isOverlapping(self):
        """ """
        # self.assertEqual(self.is_overlapping(), None)

    def test_alignVertices(self):
        """ """
        # self.assertEqual(self.align_vertices(), None)


# -----------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main(exit=False)


# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------



# !/usr/bin/python
# coding=utf-8
try:
    import pymel.core as pm
except ImportError as error:
    print(__file__, error)

# from this package:
from mayatk import core_utils


class CamUtils(object):
    """ """

    @staticmethod
    @core_utils.CoreUtils.undo
    def group_cameras(
        name="cameras", non_default=True, root_only=False, hide_group=False
    ):
        """Group cameras in the scene based on the provided parameters.

        Parameters:
                name (str): The name of the group that will contain the cameras. Default is 'cameras'.
                non_default (bool, optional): If True, only non-default cameras will be grouped. Default is True.
                root_only (bool, optional): If True, only root-level cameras (not parented to other objects) will be grouped. Default is False.
                hide_group (bool, optional): If True, the newly created group will be hidden. Default is False.

        Returns:
                PyNode: The created group node containing the cameras.
        """
        if pm.objExists(name):  # Check if the group already exists
            pm.error(f"Group '{name}' already exists.")
            return

        # Create the group and set it's visibility
        group = pm.group(empty=True, name=name)
        group.visibility.set(not hide_group)

        default_cameras = (
            "side",
            "front",
            "top",
            "persp",
            "back",
            "bottom",
            "left",
            "alignToPoly",
        )  # List of default cameras
        all_cameras = pm.ls(type="camera")  # Get all cameras in the scene
        all_camera_transforms = [
            cam.get_parent() for cam in all_cameras
        ]  # Get the parent transform nodes of the cameras

        if root_only:  # Filter cameras based on the root_only flag
            all_camera_transforms = [
                cam for cam in all_camera_transforms if cam.get_parent() is None
            ]

        # Parent cameras to the group based on the non_default flag
        for cam in all_camera_transforms:
            if non_default:
                if not any(
                    [cam.name().endswith(def_cam) for def_cam in default_cameras]
                ):
                    pm.parent(cam, group)
            else:
                pm.parent(cam, group)

        return group

    @classmethod
    def toggle_safe_frames(cls):
        """Toggle display of the film gate for the current camera."""
        camera = cls.get_current_cam()

        state = pm.camera(camera, q=True, displayResolution=1)
        if state:
            pm.camera(
                camera,
                edit=1,
                displayFilmGate=False,
                displayResolution=False,
                overscan=1.0,
            )
        else:
            pm.camera(
                camera,
                edit=1,
                displayFilmGate=False,
                displayResolution=True,
                overscan=1.3,
            )

    @staticmethod
    def get_current_cam():
        """Get the currently active camera."""
        from maya.OpenMaya import MDagPath
        from maya.OpenMayaUI import M3dView

        view = M3dView.active3dView()
        cam = MDagPath()
        view.getCamera(cam)
        camPath = cam.fullPathName()
        return camPath

    @staticmethod
    @core_utils.CoreUtils.undo
    def create_camera_from_view(name="camera#"):
        """Create a new camera based on the current view."""
        # Find the current modelPanel (viewport)
        current_panel = None
        for panel in core_utils.CoreUtils.get_panel(all=True):
            if core_utils.CoreUtils.get_panel(typeOf=panel) == "modelPanel":
                current_panel = panel
                break

        if current_panel:
            if core_utils.CoreUtils.get_panel(typeOf=current_panel) == "modelPanel":
                camera = pm.modelPanel(current_panel, q=1, cam=1)
                new_camera = pm.duplicate(camera)[0]
                pm.showHidden(new_camera)
                new_camera = pm.rename(new_camera, name)
                print(f"# Result: {new_camera} #")
                return new_camera
        else:
            print("No modelPanel found")

    @staticmethod
    @core_utils.CoreUtils.undo
    def adjust_camera_clipping(camera=None, near_clip=None, far_clip=None, auto=False):
        """Adjusts the near and far clipping planes of one or multiple cameras in Autodesk Maya.

        Parameters:
            camera (str/list/optional): The camera or list of cameras to adjust. If None, adjusts all cameras in the scene.
            near_clip (float/optional): The value to set for the near clipping plane. If None, it will not be changed.
            far_clip (float/optional): The value to set for the far clipping plane. If None, it will not be changed.
            auto (bool/optional): If True, automatically sets the near and far clipping based on the scene geometry.

        Examples:
            adjust_camera_clipping(near_clip=0.2, far_clip=1000)
            adjust_camera_clipping("persp", near_clip=0.2, far_clip=1000)
            adjust_camera_clipping(["persp", "top"], near_clip=0.2, far_clip=1000)
            adjust_camera_clipping(auto=True)
        """
        # Automatic setting based on all geometry in the scene
        if auto:
            all_geo = pm.ls(dag=True, long=True, geometry=True)
            if not all_geo:
                raise ValueError(
                    "No geometry found in the scene for automatic clipping adjustment."
                )
            bbox = pm.exactWorldBoundingBox(all_geo)

            size = [bbox[i + 3] - bbox[i] for i in range(3)]
            max_size = max(size)

            near_clip = 0.0001 * max_size
            far_clip = 5.0 * max_size  # Further increased multiplier for more room

        # Resolve the list of target cameras
        target_cameras = pm.ls(camera) if camera else pm.ls(dag=True, cameras=True)
        target_cameras = [
            cam
            for cam in target_cameras
            if "startupCameras" not in pm.listRelatives(cam, parent=True)[0].longName()
        ]

        # Loop through target cameras and adjust their clipping planes
        for cam in target_cameras:
            if near_clip:
                pm.setAttr(f"{cam}.nearClipPlane", near_clip)

            if far_clip:
                pm.setAttr(f"{cam}.farClipPlane", far_clip)


# --------------------------------------------------------------------------------------------

if __name__ == "__main__":
    pass

# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------
